package nl.sbdeveloper.mctpaudio;

import co.aikar.commands.PaperCommandManager;
import nl.sbdeveloper.mctpaudio.commands.MCTPAudioCMD;
import nl.sbdeveloper.mctpaudio.listener.LogoutListener;
import nl.sbdeveloper.mctpaudio.listener.WGListener;
import nl.sbdeveloper.mctpaudio.managers.WGManager;
import nl.sbdeveloper.mctpaudio.radio.Playlist;
import nl.sbdeveloper.mctpaudio.socket.Client;
import org.bukkit.Bukkit;
import org.bukkit.plugin.Plugin;
import org.bukkit.plugin.java.JavaPlugin;

/* Copyright (C) McThemeParks - All Rights Reserved
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 * Written by Stijn Bannink <stijnbannink23@gmail.com>, July 2020
 */

public final class MCTPAudio extends JavaPlugin {
    private static Plugin instance;

    private static PaperCommandManager commandManager;

    private static Client client;
    private static Playlist playlist;

    public void onEnable() {
        Bukkit.getLogger().info("[MCTPAudio] Loading...");

        if (!setupPlugins()) {
            Bukkit.getLogger().severe("[MCTPAudio] WorldGuard not found! Disabling...");
            getServer().getPluginManager().disablePlugin(this);
            return;
        }

        instance = this;

        getConfig().addDefault("Regions.demosound", "https://audiopagina.mcthemeparks.eu/gallery/voletarium.mp3");
        getConfig().addDefault("HueRegions.demosound", "255_0_0_254");
        getConfig().options().copyDefaults(true);
        saveConfig();

        Bukkit.getLogger().info("[MCTPAudio] Connecting with socket...");
        client = new Client("ws://173.249.31.58:8166");
        client.connect();

        Bukkit.getLogger().info("[MCTPAudio] Loading commands and events...");
        commandManager = new PaperCommandManager(this);
        commandManager.enableUnstableAPI("help");

        commandManager.registerCommand(new MCTPAudioCMD());

        Bukkit.getPluginManager().registerEvents(new WGListener(), this);
        Bukkit.getPluginManager().registerEvents(new LogoutListener(), this);

        Bukkit.getLogger().info("[MCTPAudio] Loading playlist...");
        playlist = new Playlist();

        Bukkit.getLogger().info("[MCTPAudio] Plugin is enabled!");
    }

    public void onDisable() {
        client.disconnect();
        instance = null;
        Bukkit.getLogger().info("[MCTPAudio] Plugin is disabled!");
    }

    private boolean setupPlugins() {
        if (Bukkit.getPluginManager().getPlugin("WorldGuard") != null) {
            WGManager.setWorldGuard(getServer().getPluginManager().getPlugin("WorldGuard"));
            return true;
        }
        return false;
    }

    public static Plugin getPlugin() {
        return instance;
    }

    public static Client getClient() {
        return client;
    }

    public static Playlist getPlaylist() {
        return playlist;
    }

    public static PaperCommandManager getCommandManager() {
        return commandManager;
    }
}
