package nl.sbdeveloper.mctpaudio.commands;

import co.aikar.commands.BaseCommand;
import co.aikar.commands.CommandHelp;
import co.aikar.commands.annotation.*;
import nl.sbdeveloper.mctpaudio.MCTPAudio;
import nl.sbdeveloper.mctpaudio.api.AudioType;
import nl.sbdeveloper.mctpaudio.api.HueType;
import nl.sbdeveloper.mctpaudio.managers.PinManager;
import nl.sbdeveloper.mctpaudio.utils.HeadUtil;
import nl.sbdeveloper.mctpaudio.utils.SpigotPlayerSelector;
import org.bukkit.Bukkit;
import org.bukkit.ChatColor;
import org.bukkit.command.CommandSender;
import org.bukkit.entity.Player;
import org.json.simple.JSONObject;

import java.util.ArrayList;
import java.util.List;

/* Copyright (C) McThemeParks - All Rights Reserved
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 * Written by Stijn Bannink <stijnbannink23@gmail.com>, July 2020
 */

@CommandAlias("mctpaudio")
@CommandPermission("mctp.audio")
public class MCTPAudioCMD extends BaseCommand {
    @Default
    @HelpCommand
    public void onHelp(CommandSender sender, CommandHelp help) {
        help.showHelp();
    }

    @Subcommand("toggleradio")
    @Description("")
    public void toggleRadio(CommandSender sender) {
        if (MCTPAudio.getPlaylist().isRunning()) {
            MCTPAudio.getPlaylist().stop();
            sender.sendMessage(ChatColor.GRAY + "De auto radio is stopgezet. Zodra het huidige nummer is afgelopen, gebeurt er niks meer.");
        } else {
            MCTPAudio.getPlaylist().start();
            sender.sendMessage(ChatColor.GRAY + "De auto radio is weer gestart.");
        }
    }

    @Subcommand("addsong")
    @Description("")
    public void onAddSong(CommandSender sender, String url) {
        List<String> urls = MCTPAudio.getPlugin().getConfig().getStringList("RadioSongs");
        urls.add(url);
        MCTPAudio.getPlugin().getConfig().set("RadioSongs", urls);
        MCTPAudio.getPlugin().saveConfig();
        MCTPAudio.getPlaylist().addSong(url);
        sender.sendMessage(ChatColor.GRAY + "Nummer toegevoegd aan de lijst.");
    }

    @Subcommand("play")
    @Description("")
    public void onPlay(CommandSender sender, String selector, AudioType type, String url) {
        ArrayList<Player> players = new ArrayList<>();
        Player target = Bukkit.getPlayer(selector);
        if (target != null) {
            if (!PinManager.hasPin(target.getUniqueId())) {
                sender.sendMessage(ChatColor.GRAY + "Die speler is niet verbonden met de client.");
                return;
            }

            players.add(target);
        } else {
            SpigotPlayerSelector sel = new SpigotPlayerSelector(selector);
            players.addAll(sel.getPlayers(sender));
        }

        JSONObject data;
        for (Player p : players) {
            data = new JSONObject();

            if (!PinManager.hasPin(p.getUniqueId())) continue;

            data.put("task", type.name());
            data.put("path", url);
            data.put("uuid", p.getUniqueId().toString().replace("-", ""));
            MCTPAudio.getClient().sendData(data);
        }

        sender.sendMessage(ChatColor.GRAY + "Gestart met afspelen!");
    }

    @Subcommand("hue")
    @Description("")
    public void onHue(CommandSender sender, String selector, int r, int g, int b, HueType type, @Optional Integer brightness) {
        if (brightness != null && (brightness < 0 || brightness > 254)) {
            sender.sendMessage(ChatColor.GRAY.toString() + brightness + " is geen correcte brightness.");
            return;
        }

        ArrayList<Player> players = new ArrayList<>();
        Player target = Bukkit.getPlayer(selector);
        if (target != null) {
            if (!PinManager.hasPin(target.getUniqueId())) {
                sender.sendMessage(ChatColor.GRAY + "Die speler is niet verbonden met de client.");
                return;
            }

            players.add(target);
        } else {
            SpigotPlayerSelector sel = new SpigotPlayerSelector(selector);
            players.addAll(sel.getPlayers(sender));
        }

        //CHECK FOR THE REGION SELECTOR -> Then save
        if (selector.startsWith("@a") && HeadUtil.getArgument(selector, "region").length() != 0) {
            String regionID = HeadUtil.getArgument(selector, "region");
            String data = r + "_" + g + "_" + b + "_" + type.name();
            if (brightness != null) data += "_" + brightness;

            MCTPAudio.getPlugin().getConfig().set("HueRegions." + regionID, data);
            MCTPAudio.getPlugin().saveConfig();
        }

        JSONObject data;
        for (Player p : players) {
            data = new JSONObject();

            if (!PinManager.hasPin(p.getUniqueId())) continue;

            data.put("task", "HUE");
            data.put("rgb", r + ":" + g + ":" + b);
            data.put("type", type.name());
            if (brightness != null) data.put("brightness", brightness);
            data.put("uuid", p.getUniqueId().toString().replace("-", ""));
            MCTPAudio.getClient().sendData(data);
        }

        sender.sendMessage(ChatColor.GRAY + "Indien de speler(s) is/zijn verbonden met Philips Hue, is de kleur veranderd.");
    }

    @Subcommand("setregion")
    @Description("")
    public void onSetRegion(CommandSender sender, String regionName, String url) {
        MCTPAudio.getPlugin().getConfig().set("Regions." + regionName, url);
        MCTPAudio.getPlugin().saveConfig();

        sender.sendMessage(ChatColor.GRAY + "De region zal vanaf nu muziek afspelen.");
    }

//    @Subcommand("sethueregion")
//    @Description("")
//    public void onSetHueRegion(CommandSender sender, String regionName, String url) {
//        MCTPAudio.getPlugin().getConfig().set("HueRegions." + regionName, url);
//        MCTPAudio.getPlugin().saveConfig();
//
//        sender.sendMessage(ChatColor.GRAY + "De region zal vanaf nu licht aanpassen.");
//    }

    @Subcommand("stop")
    @Description("")
    public void onStop(CommandSender sender, String selector) {
        ArrayList<Player> players = new ArrayList<>();
        Player target = Bukkit.getPlayer(selector);
        if (target != null) {
            if (!PinManager.hasPin(target.getUniqueId())) {
                sender.sendMessage(ChatColor.GRAY + "Die speler is niet verbonden met de client.");
                return;
            }

            players.add(target);
        } else {
            SpigotPlayerSelector sel = new SpigotPlayerSelector(selector);
            players.addAll(sel.getPlayers(sender));
        }

        JSONObject data;
        for (Player p : players) {
            data = new JSONObject();

            if (!PinManager.hasPin(p.getUniqueId())) continue;

            data.put("task", "STOP");
            data.put("uuid", p.getUniqueId().toString().replace("-", ""));
            MCTPAudio.getClient().sendData(data);
        }

        sender.sendMessage(ChatColor.GRAY + "Gestopt met afspelen!");
    }
}
